@@ -1,7 +1,8 @@
 from tempfile import TemporaryDirectory
 from threading import Thread
 
-from PyQt5.QtWidgets import QGridLayout, QWidget, QApplication, QSplitter, QFrame
+from PyQt5.QtWidgets import (QGridLayout, QWidget, QApplication, QSplitter,
+    QFrame)
 from PyQt5.QtCore import Qt
 from circleguard import Mod, KeylessCircleguard
 from slider import Library, Beatmap
@@ -52,9 +53,15 @@ class Interface(QWidget):
             # this logic is now duplicated here and in circlecore
             self.beatmap = self.library.lookup_by_id(beatmap_info.map_id, download=True, save=True)
 
+        dt_enabled = any(Mod.DT in replay.mods for replay in replays)
+        ht_enabled = any(Mod.HT in replay.mods for replay in replays)
+        if dt_enabled:
+            start_speed = 1.5
+        if ht_enabled:
+            start_speed = 0.75
 
-        self.renderer = Renderer(self.beatmap, replays, events, library, \
-            start_speed, paint_info, statistic_functions)
+        self.renderer = Renderer(self.beatmap, replays, events, start_speed,
+            paint_info, statistic_functions)
         self.renderer.update_time_signal.connect(self.update_slider)
         # if the renderer wants to pause itself (eg when the playback hits the
         # end of the replay), we kick it back to us (the `Interface`) so we can
@@ -180,11 +187,12 @@ class Interface(QWidget):
             replay_info.show()
         else:
             ur, frametime, snaps, edge_hits = self.replay_statistics_precalculated[replay]
-            replay_info = ReplayInfo(replay, self.beatmap, self.library.path, ur, frametime, snaps, edge_hits, self.snaps_args)
+            replay_info = ReplayInfo(replay, self.library.path, ur, frametime,
+                snaps, edge_hits, self.snaps_args)
             replay_info.seek_to.connect(self.renderer.seek_to)
 
         # don't show two of the same info panels at once
-        if self.current_replay_info != None:
+        if self.current_replay_info is not None:
             # if they're the same, don't change anything
             if replay_info == self.current_replay_info:
                 return
