@@ -1,11 +1,13 @@
 import math
 import threading
 from datetime import timedelta
+from dataclasses import dataclass
 
 import numpy as np
-from PyQt5.QtGui import QBrush, QPen, QColor, QPalette, QPainter, QPainterPath
+from PyQt5.QtGui import (QBrush, QPen, QColor, QPalette, QPainter, QPainterPath,
+    QCursor)
 from PyQt5.QtWidgets import QFrame
-from PyQt5.QtCore import Qt, pyqtSignal, QTimer, QPointF, QRectF
+from PyQt5.QtCore import Qt, pyqtSignal, QTimer, QPointF, QRectF, QRect
 from slider.beatmap import Circle, Slider, Spinner
 from slider.mod import circle_radius, od_to_ms
 from circleguard import Mod, Key
@@ -35,8 +37,6 @@ GAMEPLAY_PADDING_HEIGHT = 48 + 20
 GAMEPLAY_WIDTH = 512
 GAMEPLAY_HEIGHT = 384
 
-FRAMETIME_STEPS = 3
-FRAMETIME_FRAMES = 120
 SLIDER_TICKRATE = 50
 
 
@@ -44,10 +44,11 @@ class Renderer(QFrame):
     update_time_signal = pyqtSignal(int)
     pause_signal = pyqtSignal()
 
-    def __init__(self, beatmap, replays, events, library, start_speed, \
-        paint_info, statistic_functions):
+    def __init__(self, beatmap, replays, events, start_speed, paint_info, \
+        statistic_functions):
         super().__init__()
-        self.setMinimumSize(GAMEPLAY_WIDTH + GAMEPLAY_PADDING_WIDTH*2, GAMEPLAY_HEIGHT + GAMEPLAY_PADDING_HEIGHT*2)
+        self.setMinimumSize(GAMEPLAY_WIDTH + GAMEPLAY_PADDING_WIDTH * 2,
+            GAMEPLAY_HEIGHT + GAMEPLAY_PADDING_HEIGHT * 2)
         self.beatmap = beatmap
         # list of timestamps to highlight the frames of in a different color
         self.events = events
@@ -61,12 +62,22 @@ class Renderer(QFrame):
         self.scale = 1
         self.x_offset = 0
         self.y_offset = 0
+        # a map of QRect to Player, where the rectangle is the location of the
+        # player's info on the screen. Updated every frame (even though it's
+        # currently static except for the width, it may differ from frame to
+        # frame in the future)
+        self.player_info_positions = {}
+        # players that have been disabled by the users and we don't want to
+        # draw cursor movements for
+        self.disabled_players = []
+
+        self.setMouseTracking(True)
 
         # beatmap init stuff
         self.hitobjs_to_draw = []
 
-        self.use_hr = any([Mod.HR in replay.mods for replay in replays])
-        self.use_ez = any([Mod.EZ in replay.mods for replay in replays])
+        self.use_hr = any(Mod.HR in replay.mods for replay in replays)
+        self.use_ez = any(Mod.EZ in replay.mods for replay in replays)
         if beatmap:
             self.hit_objects = beatmap.hit_objects(hard_rock=self.use_hr, easy=self.use_ez)
             self.playback_end = self.get_hit_endtime(self.hit_objects[-1])
@@ -305,6 +316,9 @@ class Renderer(QFrame):
         Arguments:
             Player player: player to draw the cursor of.
         """
+        # don't draw anything if the player is disabled
+        if player in self.disabled_players:
+            return
         alpha_step = 1 / self.num_frames_on_screen
         pen = player.pen
         width = WIDTH_LINE_RAW_VIEW if self.raw_view else WIDTH_LINE
@@ -373,22 +387,86 @@ class Renderer(QFrame):
         PEN_WHITE.setWidth(1)
         self.painter.setPen(PEN_WHITE)
         self.painter.setOpacity(1)
-        self.painter.drawText(5, y, f"{round(self.clock.get_time())} ms")
+        ms = round(self.clock.get_time())
+        text = f"{ms}"
+        self.painter.drawText(5, y, text)
+        # we don't use a monospaced font, so our ms text may vary by as much as
+        # 10 pixels in width (possibly more, haven't checked rigorously). If
+        # we just drew our minute:seconds text directly after it, the position
+        # of that text would constantly flicker left and right, since the ms
+        # text (and thus its width) changes every single frame. To fix this,
+        # only increment widths in multiples of 10. (This will not fix the issue
+        # if the text width happens to hover exactly around a multiple of 10,
+        # but there's not much to be done in that case).
+        text_width = self.painter.boundingRect(5, y, 0, 0, 0, text).width()
+        if text_width < 50:
+            x = 50
+        elif text_width < 60:
+            x = 60
+        elif text_width < 70:
+            x = 70
+        elif text_width < 80:
+            x = 80
+        else:
+            # something crazy is going on, give up and just use text_width
+            x = text_width
 
+        # now some dirty code to deal with negattive times
+        minutes = int(ms / (1000 * 60))
+        seconds = ms // 1000
+        seconds_negative = seconds < 0
+        # pytohn modulo returns positive even when ``seconds_total`` is negative
+        seconds = seconds % 60
+        if seconds_negative:
+            # ``seconds`` can be 0 and 59 but not 60, so use 59 instead of 60
+            seconds = 59 - seconds
+        sign = ""
+        if minutes < 0 or seconds_negative:
+            sign = "-"
+            minutes = abs(minutes)
+            seconds = abs(seconds)
+
+        self.painter.drawText(5 + 4 + x, y,
+            f"ms ({sign}{minutes:01}:{seconds:02})")
+
+        self.player_info_positions = {}
         if self.num_replays > 0:
             for player in self.players:
+                def _set_opacity(opacity):
+                    if player in self.disabled_players:
+                        opacity /= 2.4
+                    self.painter.setOpacity(opacity)
+
                 y += 13
                 pen = player.pen
                 self.painter.setPen(PEN_BLANK)
                 self.painter.setBrush(QBrush(pen.color()))
-                self.painter.setOpacity(1 if Key.M1 in Key(int(player.k[player.end_pos])) else 0.3)
+                _set_opacity(1 if Key.M1 in Key(int(player.k[player.end_pos])) else 0.3)
                 self.painter.drawRect(5, y - 9, 10, 10)
-                self.painter.setOpacity(1 if Key.M2 in Key(int(player.k[player.end_pos])) else 0.3)
+                _set_opacity(1 if Key.M2 in Key(int(player.k[player.end_pos])) else 0.3)
                 self.painter.drawRect(18, y - 9, 10, 10)
-                self.painter.setOpacity(1)
+                _set_opacity(1)
                 self.painter.setPen(pen)
-                self.painter.drawText(31, y, f"{player.username} {player.mods.short_name()}: {player.xy[player.end_pos][0]:.2f}, {player.xy[player.end_pos][1]:.2f}")
+                info_text = (f"{player.username} {player.mods.short_name()}: "
+                    f"{player.xy[player.end_pos][0]:.2f}, {player.xy[player.end_pos][1]:.2f}")
+                self.painter.drawText(31, y, info_text)
+                # not sure why we need to do ``y - 9`` instead of 9 here,
+                # our ``drawText`` call is perfectly happy to accept ``y`` but
+                # we need to pass ``y - 9`` to our ``drawRect`` calls...maybe 9
+                # was a manually determined number that causes the text to align
+                # with the drawn boxes?
+                info_pos = self.painter.boundingRect(5, y - 9, 0, 0, 0, info_text)
+                info_pos = Rect(info_pos.x(), info_pos.y(), info_pos.width(), info_pos.height())
+                # unfortunately the rects overlap if we don't make this manual
+                # adjustment; would like to figure out why but this works for
+                # now.
+                info_pos.height -= 3
+                # our bounding rect starts at 5 but the text starts at 31, so
+                # we need to increase the width by the difference to account
+                info_pos.width += 31 - 5
+                self.player_info_positions[info_pos] = player
 
+            self.painter.setOpacity(1)
             self.painter.setPen(PEN_WHITE)
             if self.num_replays == 2:
                 try:
@@ -551,7 +629,8 @@ class Renderer(QFrame):
         PEN_WHITE.setWidth(self.scaled_number(WIDTH_CIRCLE_BORDER / 2))
         self.painter.setPen(PEN_WHITE)
         self.painter.setOpacity(opacity)
-        self.painter.drawEllipse(self.scaled_point(GAMEPLAY_WIDTH / 2, GAMEPLAY_HEIGHT / 2), self.scaled_number(radius), self.scaled_number(radius))
+        self.painter.drawEllipse(self.scaled_point(GAMEPLAY_WIDTH / 2, GAMEPLAY_HEIGHT / 2),
+            self.scaled_number(radius), self.scaled_number(radius))
 
     def draw_approachcircle(self, hitobj):
         """
@@ -640,14 +719,14 @@ class Renderer(QFrame):
         self.painter.drawPath(loading_bar)
 
     def draw_loading_screen(self):
-        self.painter.drawText(self.width() / 2 - 75, self.height() / 2 - 10, f"Calculating Sliders, please wait...")
+        self.painter.drawText(self.width() / 2 - 75, self.height() / 2 - 10, "Calculating Sliders, please wait...")
         self.draw_progressbar(int((self.sliders_current / self.num_sliders) * 100))
 
     def process_sliders(self):
         for i, hitobj in enumerate(self.hit_objects):
             self.sliders_current = i
             if isinstance(hitobj, Slider):
-                steps = max(2, int((self.get_hit_endtime(hitobj) - self.get_hit_time(hitobj))/SLIDER_TICKRATE))
+                steps = max(2, int((self.get_hit_endtime(hitobj) - self.get_hit_time(hitobj)) / SLIDER_TICKRATE))
                 hitobj.slider_body = [hitobj.curve(i / steps) for i in range(steps + 1)]
 
     def search_nearest_frame(self, reverse=False):
@@ -719,6 +798,31 @@ class Renderer(QFrame):
 
         self.seek_to(self.clock.time_counter + delta)
 
+    def mouseMoveEvent(self, event):
+        any_inside = False
+        for rect in self.player_info_positions:
+            qrect = rect.toQRect()
+            if qrect.contains(event.pos()):
+                any_inside = True
+                self.setCursor(QCursor(Qt.PointingHandCursor))
+        if not any_inside:
+            self.setCursor(QCursor(Qt.ArrowCursor))
+        return super().mouseMoveEvent(event)
+
+    def mousePressEvent(self, event):
+        for rect in self.player_info_positions:
+            qrect = rect.toQRect()
+            if qrect.contains(event.pos()):
+                player = self.player_info_positions[rect]
+                # toggle its membership in disabled_players, so users can click
+                # a second time to re-enable a player
+                if player in self.disabled_players:
+                    self.disabled_players.remove(player)
+                else:
+                    self.disabled_players.append(player)
+                self.update()
+        return super().mousePressEvent(event)
+
     def get_hit_endtime(self, hitobj):
         return hitobj.end_time.total_seconds() * 1000 if not isinstance(hitobj, Circle) else self.get_hit_time(hitobj)
 
@@ -780,7 +884,24 @@ class Renderer(QFrame):
             # cs doesn't matter to us if we don't have a beatmap (and we don't
             # have the attributes necessary to compute it anyway)
             return
-        use_hr = True if new_value == "HR" else False
-        use_ez = True if new_value == "EZ" else False
+        use_hr = new_value == "HR"
+        use_ez = new_value == "EZ"
         self.hitcircle_radius = circle_radius(self.beatmap.cs(hard_rock=use_hr, easy=use_ez))
         self.update()
+
+
+# not sure why dataclass won't generate a hash method for us automatically,
+# we're not using anything mutable, just ints
+@dataclass(unsafe_hash=True)
+class Rect:
+    """
+    A dataclass which mimics ``QRect`` and only serves as a hashable liaison of
+    ``QRect``.
+    """
+    x: int
+    y: int
+    width: int
+    height: int
+
+    def toQRect(self):
+        return QRect(self.x, self.y, self.width, self.height)
